<# ***************************************************************
Name:		Enable-dvnArchiveMailbox
Version:	1.0.01.15.2015
Author:     Jeff Brown

 Version					Author
 1.0.07142014				Jeff Brown
	Original code adapted from the W7EnableArchiveMailbox.ps1 created by Clifton Leonard
	Notable Changes:
		-Removed numConvert, verifyQuotas, updateQuotas functions - now just set the quotas
		 without checking for previous values
		-Removed writing to a log file - now just write to screen
		-Use Get-MailboxDatabase with filters to find specific versions of archive mailboxes
		 instead of hard coding a list of all the databases
		-If Exchange PSSession is not found, it will launch and prompt for credentials instead
		 of exiting script

 1.0.1072014			    Jeff Brown
	Added color coding for success, failed, and info messages
	Added line breaks to separate when doing bulk archive creations
	Added check at end of script to remove PSSession if it was created

 1.0.01.15.2015             Jeff Brown
    Update to new Devon Function naming convention
    Changed PS Exchange connection test to not try and establish one
    if it's not available
****************************************************************** #>

Function Enable-dvnArchiveMailbox()
{

<#
.SYNOPSIS
	Creates Exchange archive mailbox and sets archive quota to 25GB

.PARAMETER MailboxName
	MailboxName can either be the alias or email address of the Mailbox
	This parameter is optional, the script will prompt for value if not provided

.EXAMPLE
	dvn.Enable-ArchiveMailbox
	This command will prompt for the mailbox to enable with an archive mailbox
	
.EXAMPLE
	dvn.Enable-ArchiveMailbox -MailboxName John.Smith@dvn.com
	This command will enable 'John.Smith@dvn.com' with an archive mailbox
	The mailbox name can either be the email address or the alias of the user

.NOTES
	Use of this function requires Exchange permissions to the following cmdlets:
	Get-User, Get-Mailbox, Get-MailboxDatabase, Enable-Mailbox
	
	Version 1.0.1072014
#>

#**********   Parameters   **********

param(
	[Parameter(Position=0, Mandatory=$false)]
	$MailboxName
)

#**********   Variables   **********

# Mailbox Quotas (some may be temporary)
$IssueWarningQuota 			= 50GB 
$ProhibitSendQuota 			= 50GB
$ArchiveWarningQuota 		= 20GB 
$ArchiveQuota 				= 25GB
$ManagedFolderMailboxPolicy = $Null
$ProhibitSendReceiveQuota 	= "unlimited"	# Shouldn't be adjusted during archive process
$PowerShellVirtualDirectory	= "http://odcmsmxex201p.corp.dvn.com/PowerShell"
$EnableArchiveMB			= $false
$AllMBInfo					= ""
$DomainController			= ""
$errorText					= "white"
$errorBackground			= "darkred"
$success					= "green"
$info						= "yellow"

# Default stature for errors returned when executing remote commands
$EAPreference = "SilentlyContinue"

# Default domain to query/execute
$domain = "corp.dvn.com"

#**********   Main Code   **********

Write-Host

# Verify PS Connection to Exchange
$connExchange = $False	
$PSSessions = Get-PSSession | select ConfigurationName,Availability
foreach ($PSSession in $PSSessions){
	If ($PSSession.ConfigurationName -ilike "Microsoft.Exchange")
    {
		$connExchange = $True
	}
}

# If no PSSession found for Exchange, exit function
If ($connExchange -eq $False)
{
	Write-Host "`tERROR: Current Remote PS Session to Exchange NOT Detected" -ForegroundColor $errorText -BackgroundColor $errorBackground
	Write-Host "`tPlease create remote PS Session to Exchange before continuing" -ForegroundColor $info
    Write-Host "`tExiting Script..." -ForegroundColor $info
    RETURN
}

# Loop until user enters name
If (($MailboxName -eq "") -or ($MailboxName -eq $Null)){
	Do {$MailboxName = Read-Host "`tEnter name of mailbox to archive enable (eg, 'JohnD', 'John.Doe@dvn.com')"}
	Until ($MailboxName -ne "")
}

# Retrieves a random domain controller for sending all requests & submissions
$dirContext = New-Object System.DirectoryServices.ActiveDirectory.DirectoryContext("domain",$domain)
$DCs = [System.DirectoryServices.ActiveDirectory.Domain]::GetDomain($dirContext).domainControllers | ?{$_.Name -ilike "ODC*"}
$DCproperties = ($DCs)[(Get-Random -maximum ($DCs.count))]
$DomainController = $DCproperties.Name

# Verify User and Mailbox Exist
$UserName = Get-User -Identity $MailboxName -ErrorAction $EAPreference -DomainController $DomainController | select Name
If ($UserName -ne $Null){
	Write-Host "`tLocated User Account for: $MailboxName"
	$AllMBInfo = Get-Mailbox -Identity $MailboxName -ErrorAction $EAPreference -DomainController $DomainController
	If ($AllMBInfo.Name -ne $Null){
		Write-Host "`tLocated Primary Mailbox for: $MailboxName"
		If ($AllMBInfo.ArchiveName -ne $Null){
			Write-Host "`tWARNING: Located prior Archive Mailbox for: $MailboxName -- Exiting Script" -ForegroundColor $info
		}
		Else{
			Write-Host "`tNo Archive Mailbox found for: $MailboxName -- Will Provision" -ForegroundColor $info
			$EnableArchiveMB = $true
		}
	}
	Else{
		Write-Host "`tERROR: No Mailbox found for: $MailboxName -- Exiting Script" -ForegroundColor $errorText -BackgroundColor $errorBackground
	}
}
Else{
	Write-Host "`tERROR: Unable to locate User Account for: $MailboxName -- Exiting Script" -ForegroundColor $errorText -BackgroundColor $errorBackground
}

If ($EnableArchiveMB -eq $true){
	# Setting Default Quotas on Mailbox
	Write-Host "`tSetting Archive Quotas on Mailbox: $MailboxName"
	Set-Mailbox -Identity $MailboxName -IssueWarningQuota $IssueWarningQuota -ProhibitSendQuota $ProhibitSendQuota -ArchiveWarningQuota $ArchiveWarningQuota `
		-ArchiveQuota $ArchiveQuota -DomainController $DomainController

	# Checks for Exchange version and enables archive on appropriate servers
	# Additional If statements will be added if archives are still used on future versions of Exchange
	If ($AllMBInfo.AdminDisplayVersion -like "Version 15.*"){
		# Find All Exchange 2013 Archive Databases
		$ArchiveMBDBs = @(Get-MailboxDatabase *ADB* | Where-Object {$_.AdminDisplayVersion -like "Version 15.*"} | foreach {$_.Name})
		
		# Picks random DB to create archive on
		$archiveDB = Get-Random $ArchiveMBDBs
		$Null = Enable-Mailbox -Identity $MailboxName -Archive -ArchiveDatabase $archiveDB -DomainController $DomainController
		
		# Waits for replication
		Start-Sleep -Seconds 4
		
		# Checks if archive was successfully created
		$ArchiveName = Get-Mailbox -Identity $MailboxName -DomainController $DomainController | Select archiveName
		If (($ArchiveName -ne $Null) -or ($ArchiveName -ne "")){
			Write-Host "`tSUCCESS: Successfully Provisioned Archive Mailbox for: $MailboxName" -ForegroundColor $success
		} Else {
			Write-Host "`tERROR: Archive Mailbox was not provisioned for: $MailboxName" -ForegroundColor $errorText -BackgroundColor $errorBackground
		}
	}
	Else{
		Write-Host "`tERROR: User Mailbox not on supported Exchange Version for archives" -ForegroundColor $errorText -BackgroundColor $errorBackground
		Write-Host "`tCurrent User Mailbox Exchange Version is: " $AllMBInfo.AdminDisplayVersion -ForegroundColor $errorText -BackgroundColor $errorBackground
	}
}

# Clear variables
$connExchange				= $Null
$MailboxName				= $Null
$EnableArchiveMB			= $Null
$AllMBInfo					= $Null
$DomainController			= $Null
$UserName					= $null

} # End Of Enable-dvnArchiveMailbox Function
